import { Suspense, lazy } from "react";
import { Toaster } from "@/components/ui/toaster";
import { Toaster as Sonner } from "@/components/ui/sonner";
import { TooltipProvider } from "@/components/ui/tooltip";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";
import { BrowserRouter, Routes, Route } from "react-router-dom";
import { AuthProvider } from "@/hooks/useAuth";
import { TenantProvider } from "@/hooks/useTenant";
import { WorkshopProvider } from "@/hooks/useWorkshop";
import { LanguageProvider } from "@/contexts/LanguageContext";
import { ErrorBoundary } from "@/components/common/ErrorBoundary";
import { PerformanceMonitor } from "@/components/common/PerformanceMonitor";
import { LoadingSpinner } from "@/components/common/LoadingSpinner";
import ProtectedRoute from "@/components/auth/ProtectedRoute";

// Lazy load major route components for better performance
const Index = lazy(() => import("./pages/Index"));
const Auth = lazy(() => import("./pages/Auth"));
const ResetPassword = lazy(() => import("./pages/ResetPassword"));
const Dashboard = lazy(() => import("./pages/Dashboard"));
const AdminDashboard = lazy(() => import("./pages/AdminDashboard"));
const PublicBooking = lazy(() => import("./pages/PublicBooking"));
const ClientAccount = lazy(() => import("./pages/ClientAccount"));
const ClientAppointments = lazy(() => import("./pages/ClientAppointments"));
const ClientVehicles = lazy(() => import("./pages/ClientVehicles"));
const ClientServiceHistory = lazy(() => import("./pages/ClientServiceHistory"));
const ClientQuotations = lazy(() => import("./pages/ClientQuotations"));
const ClientMessages = lazy(() => import("./pages/ClientMessages"));
const ClientDetailPage = lazy(() => import("./components/clients/ClientDetailPage"));
const EmailVerificationComplete = lazy(() => import("./components/booking/EmailVerificationComplete"));
const VerifyEmail = lazy(() => import("./pages/VerifyEmail"));
const VerifyWorkshop = lazy(() => import("./pages/VerifyWorkshop"));
const WorkshopSignup = lazy(() => import("./pages/WorkshopSignup"));
const WorkshopSignupNew = lazy(() => import("./pages/WorkshopSignupNew"));
const ClientSignup = lazy(() => import("./pages/ClientSignup"));
const NotFound = lazy(() => import("./pages/NotFound"));

// Optimized Query Client with better caching strategies
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      staleTime: 5 * 60 * 1000, // 5 minutes
      gcTime: 10 * 60 * 1000, // 10 minutes (formerly cacheTime)
      retry: (failureCount, error: any) => {
        // Don't retry on 4xx errors
        if (error?.status >= 400 && error?.status < 500) {
          return false;
        }
        return failureCount < 3;
      },
      refetchOnWindowFocus: false,
      refetchOnReconnect: true,
    },
    mutations: {
      retry: 1,
    }
  }
});

const App = () => (
  <ErrorBoundary>
    <PerformanceMonitor />
    <QueryClientProvider client={queryClient}>
      <LanguageProvider>
        <BrowserRouter>
          <AuthProvider>
            <TenantProvider>
              <WorkshopProvider>
                <TooltipProvider>
                  <Toaster />
                  <Sonner />
                  <Suspense fallback={<LoadingSpinner />}>
                    <Routes>
                      <Route path="/" element={<Index />} />
                      <Route path="/auth" element={<Auth />} />
                      <Route path="/reset-password" element={<ResetPassword />} />
          <Route 
            path="/dashboard/:section?" 
            element={
              <ProtectedRoute requiredRole="workshop" fallbackPath="/">
                <Dashboard />
              </ProtectedRoute>
            } 
          />
          <Route 
            path="/admin" 
            element={
              <ProtectedRoute requiredRole="admin" fallbackPath="/">
                <AdminDashboard />
              </ProtectedRoute>
            } 
          />
                      <Route path="/book/:slug" element={<PublicBooking />} />
                      <Route path="/verify-email/:token" element={<EmailVerificationComplete />} />
                      <Route path="/verify-email" element={<VerifyEmail />} />
                      <Route path="/verify-workshop" element={<VerifyWorkshop />} />
                      <Route path="/workshop-signup" element={<WorkshopSignup />} />
                      <Route path="/signup/workshop" element={<WorkshopSignupNew />} />
                      <Route path="/signup/client" element={<ClientSignup />} />
                      <Route path="/client-dashboard" element={<ClientAccount />} />
                      <Route path="/account" element={<ClientAccount />} />
                      <Route path="/appointments" element={<ClientAppointments />} />
                      <Route path="/messages" element={<ClientMessages />} />
                      <Route path="/vehicles" element={<ClientVehicles />} />
                      <Route path="/service-history" element={<ClientServiceHistory />} />
                      <Route path="/quotations" element={<ClientQuotations />} />
                      <Route path="/client/:clientId" element={<ClientDetailPage />} />
                      <Route path="*" element={<NotFound />} />
                    </Routes>
                  </Suspense>
                </TooltipProvider>
              </WorkshopProvider>
            </TenantProvider>
          </AuthProvider>
        </BrowserRouter>
      </LanguageProvider>
    </QueryClientProvider>
  </ErrorBoundary>
);

export default App;
